@@ -682,31 +682,30 @@ def _get_quad_slip_ds_ss(q, rake, cp, p):
     )
 
     # Need to track 'origin'
-    s0 = np.array([[0], [0], [0]])
-
+    s0 = np.array([[0.0], [0.0], [0.0]])
     # Convert from 'local' to geographic coords
     s1_ll = proj(
-        s1mat[0,],
-        s1mat[1,],
+        s1mat[0,][0],
+        s1mat[1,][0],
         reverse=True,
     )
     d1_ll = proj(
-        d1mat[0,],
-        d1mat[1,],
+        d1mat[0,][0],
+        d1mat[1,][0],
         reverse=True,
     )
     s0_ll = proj(s0[0], s0[1], reverse=True)
 
     # And then back to ECEF:
     s1_ecef = latlon2ecef(
-        s1_ll[1],
-        s1_ll[0],
-        s1mat[2,],
+        s1_ll[1][0],
+        s1_ll[0][0],
+        s1mat[2,][0],
     )
     d1_ecef = latlon2ecef(
-        d1_ll[1],
-        d1_ll[0],
-        d1mat[2,],
+        d1_ll[1][0],
+        d1_ll[0][0],
+        d1mat[2,][0],
     )
     s0_ecef = latlon2ecef(s0_ll[1], s0_ll[0], s0[2])
     s00 = s0_ecef[0].reshape(-1)
