@@ -7,6 +7,7 @@ import copy
 from enum import Enum
 
 import numpy as np
+from scipy import integrate
 from scipy.stats import norm
 import openquake.hazardlib.const as oqconst
 
@@ -230,8 +231,12 @@ class FFSimmer:
         )
 
         gd = grid.getGeoDict()
+        if gd.xmin > gd.xmax:
+            lons = np.linspace(gd.xmin, gd.xmax+360., gd.nx)
+        else:
+            lons = np.linspace(gd.xmin, gd.xmax, gd.nx)
         glon, glat = np.meshgrid(
-            np.linspace(gd.xmin, gd.xmax, gd.nx),
+            lons,
             np.linspace(gd.ymax, gd.ymin, gd.ny),
         )
         gdepth = np.zeros_like(glon)
@@ -659,8 +664,10 @@ class RandomFiniteFault(object):
         max_dip=None,
         dx_min_frac=0.0,
         dx_max_frac=1.0,
-        delta_depth=20.0,
         ztor=-1,
+        aspect_ratio=-1,
+        min_sz_depth=0,
+        max_sz_depth=-1,
         seed=None,
     ):
         """
@@ -681,12 +688,20 @@ class RandomFiniteFault(object):
                 along-strike placement of the hypocenter to from
                 dx_min_frac * length to dx_max_frac * length. (default
                 0.0/1.0)
-            delta_depth (float): The depth range of the hypocenter is
-                uniformly drawn from depth - delta_depth to
-                depth + delta_depth (kilometers) (Default: 20.0)
             ztor (float): Fix the top of rupture to a specific depth
                 (kilometers, positive down). Negative values allow the
                 TOR to float. (Default -1.)
+            aspect_ratio (float): The minimum aspect ratio (L/W) for the
+                the rupture plane. The actual aspect ratio may be greater
+                than this if the length needs to be increased to
+                accommodate the rupture area. (Default -1 means that
+                the starting aspect ratio is computed from Huang et
+                al. (2024) eqs 2 and 3.)
+            min_sz_depth (float): The minimum seismogenic depth. (Default
+                0 == the surface.)
+            max_sz_depth (float): The maximum seismogenic depth. (Default
+                -1 means that the seismogenic depth will be determined
+                by the tectonic regime.)
             seed (int): A seed for the random number generator (for testing
                 only).
         """
@@ -701,8 +716,10 @@ class RandomFiniteFault(object):
             max_dip,
             dx_min_frac,
             dx_max_frac,
-            delta_depth,
             ztor,
+            aspect_ratio,
+            min_sz_depth,
+            max_sz_depth,
         )
 
     def initialize(
@@ -713,15 +730,20 @@ class RandomFiniteFault(object):
         max_dip,
         dx_min_frac,
         dx_max_frac,
-        delta_depth,
         ztor,
+        aspect_ratio,
+        min_sz_depth,
+        max_sz_depth,
     ):
         """
         Initialize the random arrays
         """
-        (mscale, aspect, mindip_deg, maxdip_deg, szdepth, smech) = (
-            get_scaling_params(self.origin)
+        (mscale, mindip_deg, maxdip_deg, szdepth, smech) = get_scaling_params(
+            self.origin
         )
+        if max_sz_depth > 0:
+            szdepth = max_sz_depth
+
         _, _, _, _, area_mean, area_std = dimensions_from_magnitude(
             self.origin.mag, mscale, 1, 0, smech
         )
@@ -729,19 +751,22 @@ class RandomFiniteFault(object):
         depth = hypo.z
         if depth > szdepth:
             szdepth = depth
-        depth = self.rng.uniform(
-            np.maximum(depth - delta_depth, 0),
-            np.minimum(depth + delta_depth, szdepth),
-            self.nsim,
-        )
+        depth = self.rng.normal(depth, 10.0, self.nsim)
+        depth[depth < min_sz_depth] = min_sz_depth
+        depth[depth > szdepth] = szdepth
         area = 10 ** self.rng.normal(
             np.log10(area_mean[0]), area_std, self.nsim
         )
-        width = np.sqrt(area / aspect)
         if min_dip is not None:
             mindip_deg = min_dip
             maxdip_deg = max_dip
-        dip = np.radians(self.rng.uniform(mindip_deg, maxdip_deg, self.nsim))
+        nu = self.rng.uniform(mindip_deg, maxdip_deg, self.nsim)
+        dip = np.radians(nu)
+        if aspect_ratio < 0:
+            aspect = calc_asp(self.origin.mag, nu)
+        else:
+            aspect = aspect_ratio
+        width = np.sqrt(area / aspect)
         # Need to apply a max to dy that might be less than width based on the
         # eq depth.
         # height = vertical projection of plane
@@ -761,15 +786,14 @@ class RandomFiniteFault(object):
         #
         height = width * np.sin(dip)
         # Don't allow the rupture to extend below the seismogenic depth
-        height[height > szdepth] = szdepth
+        height[height > (szdepth - min_sz_depth)] = szdepth - min_sz_depth
         width = height / np.sin(dip)
-        # Limit the rupture to stay between the surface and the szdepth
+        # Limit the rupture to stay between 0 and the max_sz_depth
         # but allow it to float in between
         if ztor < 0:
-            dy = self.rng.uniform(
-                np.maximum(0, width - (szdepth - depth) / np.sin(dip)),
-                np.minimum(width, width - (height - depth) / np.sin(dip)),
-            )
+            dy_min = np.maximum(0, width - (szdepth - depth) / np.sin(dip))
+            dy_max = np.minimum(width, width - (height - depth) / np.sin(dip))
+            dy = dy_min + self.rnd_weibull(self.nsim) * (dy_max - dy_min)
         else:
             # Fix ztor to a specific depth. This is a weak contraint: there is
             # no guarantee that for a given depth, dip, and width that the TOR
@@ -778,7 +802,15 @@ class RandomFiniteFault(object):
             dy[dy > width] = width[dy > width]
 
         length = area / width
-        dx = self.rng.uniform(dx_min_frac * length, dx_max_frac * length)
+        dx_min = dx_min_frac * length
+        dx_max = dx_max_frac * length
+        if dx_min_frac == 0 and dx_max_frac == 1:
+            frac = self.rng.normal(0.5, 0.23, self.nsim)
+            frac[frac < 0] = 0
+            frac[frac > 1] = 1
+        else:
+            frac = self.rng.uniform(size=self.nsim)
+        dx = dx_min + frac * (dx_max - dx_min)
         strike = self.rng.uniform(min_strike, max_strike, self.nsim)
         hx = np.array([hypo.x])
         hy = np.array([hypo.y])
@@ -816,6 +848,46 @@ class RandomFiniteFault(object):
         """
         return self.nsim
 
+    def rnd_weibull(self, nsamp):
+        """
+        Compute random samples from the Weibull distribution using
+        inverse transform sampling.
+        """
+        A = 0.612
+        B = 3.353
+        x = np.linspace(0, 1, 1000)
+        y = weibull(A, B, x)
+        cdf = integrate.cumulative_trapezoid(y, x, initial=x[0])
+        icdf = np.interp(x, cdf, x)
+        uniform_samples = self.rng.uniform(size=nsamp)
+        rnd_array = np.interp(uniform_samples, x, icdf)
+        return rnd_array
+
+
+def calc_asp(M, nu):
+    """
+    Calculate the aspect ratio using scaling relations (EQ 2 & 3)
+    from Huang et al. (2024), given the magnitude, M, and an
+    array of dips (in degrees), nu.
+    """
+    c0 = 0.1139
+    c1 = 0.532
+    c2 = 7.17
+    c3 = -0.0105
+
+    M_BP = c2 + c3 * nu
+
+    aspect = 10 ** (c0 + c1 * (M - M_BP))
+    aspect[M <= M_BP] = 10**c0
+    return aspect
+
+
+def weibull(A, B, x):
+    """
+    The Weibull distribution.
+    """
+    return (B / A) * np.power(x / A, B - 1) * np.exp(-np.power(x / A, B))
+
 
 def get_scaling_params(origin):
     """
@@ -831,11 +903,9 @@ def get_scaling_params(origin):
         smech = Mechanism.ALL
         mindip_deg = 35.0
         maxdip_deg = 90.0
-        aspect = 1.7
         szdepth = 20.0
     elif origin.tectonic_region == "Active":
         mscale = MagScaling.HB08
-        aspect = 1.7
         szdepth = 20.0
         if mech == "ALL":
             # HB08 doesn't have an 'ALL' mechanism, so use WC94
@@ -857,7 +927,6 @@ def get_scaling_params(origin):
             maxdip_deg = 90.0
     elif origin.tectonic_region == "Stable":
         mscale = MagScaling.S14
-        aspect = 1.0
         szdepth = 15.0
         if mech == "ALL":
             smech = Mechanism.ALL
@@ -876,7 +945,6 @@ def get_scaling_params(origin):
             mindip_deg = 60.0
             maxdip_deg = 90.0
     elif origin.tectonic_region == "Subduction":
-        aspect = 1.7
         szdepth = np.inf
         if mech == "ALL":
             mscale = MagScaling.Sea10_interface
@@ -905,12 +973,11 @@ def get_scaling_params(origin):
         )
         mscale = MagScaling.WC94
         smech = Mechanism.ALL
-        aspect = 1.7
         szdepth = 20.0
         mindip_deg = 35.0
         maxdip_deg = 90.0
 
-    return (mscale, aspect, mindip_deg, maxdip_deg, szdepth, smech)
+    return (mscale, mindip_deg, maxdip_deg, szdepth, smech)
 
 
 def dimensions_from_magnitude(
